/*
   These test functions fill an array of size n, where n is a command line parameter,
   with a random permutation of the integers from 1 to n, and then the random array
   is sorted. The time it takes to sort the array is (roughly) calculated.
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>      // needed for time_t         and time()
#include <sys/time.h>  // needed for struct timeval and gettimeofday()
#include <windows.h>   // needed for LARGE_INTEGER  and QueryPerformanceCounter()
#include "Timing.h"
#include "Mergesort.h"
#include "QuickSort.h"

/*
   This function will randomly permute the elements of the array
   between index low and high.
*/
void permutation(int* a, int low, int high)
{
   int i, upperBound, temp;

   srand( time(NULL) );

   upperBound = high;
   // The array from indices low to upperBound is the "jar" to draw randomly
   // from, and the array from indices upperBound to high is the list of
   // numbers that have already been drawn out of the "jar".
   for (i = low; i <= high; i++)
   {
      // pick a slot in the "jar" part of the array
      int randomIndex = low + (rand() % (upperBound-(low-1)));

      // swap a[upperBound-1] with a[randomIndex]
      // (that is, move an item from the "jar" part of the array
      // to the list part of the array)
      temp = a[upperBound];
      a[upperBound] = a[randomIndex];
      a[randomIndex] = temp;

      // lower the upperbound
      upperBound--;  // lower the boundary of the "jar" part of the array
   }
}//permutation()


/*
   This function prints an array in rows of 10.
*/
void printTheArray(int* a, int low, int high)
{
   int i;
   char temp[4];  // assume high is less than 998
   int w = strlen(itoa(high+1, temp, 10));
   printf("Here is the array.\n");
   for (i = low; i <= high; i++)
   {
      printf(" %*d", w, a[i]);
      if ((i+1-low)%10 == 0) printf("\n");
   }
   if ((i+1-low)%10 != 1) printf("\n");
}//printTheArray()




int main(int argc, char** argv)
{
   LARGE_INTEGER timeStart, timeEnd;             // time in clock-ticks
   LARGE_INTEGER clockFrequency;                 // clock-ticks per second
   double elapsedTicks;
   TimingObject startTime, endTime, elapsedTime; // time in milli-seconds
   struct timeval startMicros, endMicros;        // time in micro-seconds
   int elapsedMicros;

   int* integerArray;
   int i;

   int n;
   if (argc > 1)
   {
      n = atoi( argv[1] );  // get parameter from command line
   }
   else
   {
      printf("Usage: Sort <n>");
      return 0;
   }

   integerArray = (int*)malloc(n*sizeof(int));  // array that will hold the permutation.
   if (integerArray == NULL)  // error check
   {
      fprintf(stderr, "Sort: not enough memory for array of size %d.\n", n);
      return -1;
   }

   // initialize the array with the numbers for 1 to n
   for (i = 0; i < n; i++) integerArray[i] = i+1;

   // the permutation() function will randomly permute
   // the contents of the array
   permutation(integerArray, 0, n-1);

   // if n is small enough, print out the array.
   if (n <= 128)
   {
      printTheArray(integerArray, 0, n-1);
   }

   // get number of clock ticks per second
   QueryPerformanceFrequency(&clockFrequency);

   // sort the array
   gettimeofday( &startMicros, NULL );    // micro-second timer
   getThreadTime( &startTime );           // milli-second timer
   QueryPerformanceCounter( &timeStart ); // clock-tick timer
     // use just one of the two sorting algorithms
     quicksort( integerArray, 0, n-1 );
//     mergeSort( integerArray, 0, n-1 );
   QueryPerformanceCounter( &timeEnd );
   getThreadTime( &endTime );
   gettimeofday( &endMicros, NULL );

   // Compute the sorting time (three ways).
   elapsedTicks = timeEnd.QuadPart - timeStart.QuadPart;
   computeElapsedTime( &startTime, &endTime, &elapsedTime );
   elapsedMicros = (endMicros.tv_sec*1000000  + endMicros.tv_usec)
                  - (startMicros.tv_sec*1000000 + startMicros.tv_usec);

   printf("It took\n");
   printf("\t%f seconds,\n", elapsedTicks/clockFrequency.QuadPart);
   printTimingObject( &elapsedTime );
   printf("\t%d micro-seconds,\n", elapsedMicros);

   printf("to sort an array of size %d.\n", n);

   // if n is small enough, print out the array.
   if (n <= 128)
   {
      printTheArray(integerArray, 0, n-1);
   }
   else // test that the array really is sorted
   {
      for (i = 0; i < n; i++)
         if (integerArray[i] != i+1)
         {
            printf("WARNING: The array is NOT sorted!\n");
            break;
         }
   }

   free(integerArray);

   return 0;
}//main()
